# 接入

Pigsty提供了丰富的接入方式，用户可以根据自己的基础设施情况与喜好自行选择接入模式。

用户可以通过多种方式访问数据库服务。

在集群层次，用户可以通过**集群域名**+服务端口的方式访问集群提供的 [**四种默认服务**](../service#默认服务)，Pigsty强烈建议使用这种方式。当然用户也可以绕开域名，直接使用集群的VIP（L2 or L4）访问数据库集群。

在实例层次，用户可以通过节点IP/域名 + 5432端口直连Postgres数据库，也可以用6432端口经由Pgbouncer访问数据库。还可以通过Haproxy经由5433~543x访问实例所属集群提供的服务。

如何访问数据库，最终取决于数据库所使用的**流量接入方案**。



## 典型接入方案

Pigsty推荐使用基于Haproxy的接入方案（1/2），在生产环境中如果有基础设施支持，也可以使用基于L4VIP（或与之等效的负载均衡服务）的接入方案（3）。

| 序号 | 方案                                | 说明                                                      |
| ---- | ----------------------------------- | --------------------------------------------------------- |
| 1    | [DNS + Haproxy](#dns-haproxy)      | 标准高可用接入方案，系统无单点。                          |
| 2    | [L2VIP + Haproxy](#l2-vip-haproxy) | Pigsty沙箱使用的标准接入架构，使用L2 VIP确保Haproxy高可用 |
| 3    | [L4VIP + Haproxy](#l4-vip-haproxy) | 方案2的变体，使用L4 VIP确保Haprxoy高可用。                |
| 4    | [L4 VIP](#l4-vip)                   | 大规模**高性能生产环境**建议使用DPVS L4 VIP直接接入       |
| 5    | [Consul DNS](#consul-dns)           | 使用Consul DNS进行服务发现，绕开VIP与Haproxy              |
| 6    | [Static DNS](#static-dns)           | 传统静态DNS接入方式                                       |
| 7    | [IP](#ip)                           | 采用智能客户端接入                                        |





## DNS + Haproxy

### 方案简介

标准高可用接入方案，系统无单点。灵活性，适用性，性能的最佳平衡点。

集群中的Haproxy采用Node Port的方式统一对外暴露 [**服务**](../service/)。每个Haproxy都是幂等的实例，提供完整的负载均衡与服务分发功能。Haproxy部署于每一个数据库节点上，因此整个集群的每一个成员在使用效果上都是幂等的。（例如访问任何一个成员的5433端口都会连接至主库连接池，访问任意成员的5434端口都会连接至某个从库的连接池）

Haproxy本身的可用性**通过幂等副本实现**，每一个Haproxy都可以作为访问入口，用户可以使用一个、两个、多个，所有Haproxy实例，每一个Haproxy提供的功能都是完全相同的。

**用户需要自行确保应用能够访问到任意一个健康的Haproxy实例**。作为最朴素的一种实现，用户可以将数据库集群的DNS域名解析至若干Haproxy实例，并启用DNS轮询响应。而客户端可以选择完全不缓存DNS，或者使用长连接并实现建立连接失败后重试的机制。又或者参考方案2，在架构侧通过额外的L2/L4 VIP确保Haproxy本身的高可用。

### 方案优越性

* 无单点，高可用

* VIP固定绑定至主库，可以灵活访问

### 方案局限性

* 多一跳

* Client IP地址丢失，部分HBA策略无法正常生效

* **Haproxy本身的高可用通过幂等副本，DNS轮询与客户端重连实现**

  DNS应有轮询机制，客户端应当使用长连接，并有建连失败重试机制。以便单Haproxy故障时可以自动漂移至集群中的其他Haproxy实例。如果无法做到这一点，可以考虑使用**接入方案2**，使用L2/L4 VIP确保Haproxy高可用。

### 方案示意

![](../_media/access-dns-ha.svg)





## L2 VIP + Haproxy

### 方案简介

Pigsty沙箱使用的标准接入方案，采用单个域名绑定至单个L2 VIP，VIP指向集群中的HAProxy。

集群中的Haproxy采用Node Port的方式统一对外暴露 [**服务**](../service/)。每个Haproxy都是幂等的实例，提供完整的负载均衡与服务分发功能。而**Haproxy本身的可用性则通过L2 VIP来保证**。

每个集群都分配有**一个**L2 VIP，固定绑定至集群主库。当主库发生切换时，该L2 VIP也会随之漂移至新的主库上。这是通过`vip-manager`实现的：`vip-manager`会查询Consul获取集群当前主库信息，然后在主库上监听VIP地址。

集群的L2 VIP有与之对应的**域名**。域名固定解析至该L2 VIP，在生命周期中不发生变化。

### 方案优越性

* 无单点，高可用

* VIP固定绑定至主库，可以灵活访问

### 方案局限性

* 多一跳

* Client IP地址丢失，部分HBA策略无法正常生效

* 所有候选主库必须**位于同一二层网络**。

  作为另一种备选变体，用户也可以通过使用L4 VIP绕开此限制，但相比L2 VIP会额外多一跳。

### 方案示意

![](../_media/access.svg)



## L4 VIP + Haproxy

### 方案简介

接入方案1/2的另一种变体，通过L4 VIP确保Haproxy的高可用

### 方案优越性

* 无单点，高可用
* 可以同时使用**所有**的Haproxy实例，均匀承载流量。
* 所有候选主库**不需要**位于同一二层网络。
* 可以操作单一VIP完成流量切换（如果同时使用了多个Haproxy，不需要逐个调整）

### 方案局限性

* 多两跳，较为浪费，如果有条件可以直接使用方案4: L4 VIP直接接入。
* Client IP地址丢失，部分HBA策略无法正常生效





## L4 VIP

### 方案简介

大规模**高性能生产环境**建议使用 L4 VIP接入（FullNAT，DPVS）

### 方案优越性

* 性能好，吞吐量大
* 可以通过`toa`模块获取正确的客户端IP地址，HBA可以完整生效。

### 方案局限性

* 仍然多一条。
* 需要依赖外部基础设施，部署复杂。
* 未启用`toa`内核模块时，仍然会丢失客户端IP地址。
* 没有Haproxy屏蔽主从差异，集群中的每个节点不再“**幂等**”。





## Consul DNS

### 方案简介

L2 VIP并非总是可用，特别是所有候选主库必须**位于同一二层网络**的要求可能不一定能满足。

在这种情况下，可以使用DNS解析代替L2 VIP，进行

### 方案优越性

* 少一跳

### 方案局限性

* 依赖Consul DNS
* 用户需要合理配置DNS缓存策略





## Static DNS

### 方案简介

传统静态DNS接入方式

### 方案优越性

* 少一跳
* 实施简单

### 方案局限性

* 没有灵活性
* 主从切换时容易导致流量损失





## IP

### 方案简介

采用智能客户端直连数据库IP接入

### 方案优越性

* 直连数据库/连接池，少一条
* 不依赖额外组件进行主从区分，降低系统复杂性。

### 方案局限性

* 灵活性太差，集群扩缩容繁琐。





